#!/boot/home/config/bin/python

import sys

# Make sure the app looks in the folders for those classes
sys.path.insert(0, 'SplitView')
sys.path.insert(0, 'CLV')
sys.path.insert(0, 'lib')

import os, pickle, thread

# Be API
import BApplication
from BBox import BBox
from BWindow import BWindow
from BScrollView import BScrollView
from BListView import BListView
from BStringItem import BStringItem
from BTextControl import BTextControl
from SplitView import Splitter
from ColumnListView import ColumnListView
from BMessage import BMessage
from BMenu import BMenu
from BMenuBar import BMenuBar
from BMenuItem import BMenuItem
from BSeparatorItem import BSeparatorItem
from BButton import BButton
from BView import BView
from BCheckBox import BCheckBox
from BAlert import BAlert

from InterfaceKit import *
from AppKit import *


class MainWindow(BWindow):
	def __init__(self, rect, title):
		BWindow.__init__(self, rect, title, B_TITLED_WINDOW, B_QUIT_ON_WINDOW_CLOSE)
		
		# Initialize the system dictionary and selected item
		self.SYSTEMS = {}
		self.SELECTED = ''
		self.optChk = 0
		self.title = title
		# Check for config file, load it or make it accordingly
		if os.path.exists('settings/systems.cfg'):
			file = open('settings/systems.cfg', 'r')
			cp = file.read()
			self.SYSTEMS = pickle.loads(cp)
		else:
			confp = pickle.dumps(self.SYSTEMS)
			file = open('settings/systems.cfg', 'w')
			file.write(confp)
			file.close()
		
		# Coords of the window for use in laying things out
		rectt = self.Bounds()
		
		# break the BRect into 4 values, left, top. right, bottom
		l, t, r, b = rectt
		
		#Build Menubar		
		self.bar = BMenuBar(rectt, 'Bar')
		Menus1 = (('File', ((1, 'Remove System', ('r', B_COMMAND_KEY)), (None, None, None), (9, 'About',(None)), (B_QUIT_REQUESTED, 'Quit',(None)))))
		
		# Add Items to Menu
		self.mkey = {}
		menu, items = Menus1
		menu = BMenu(menu)
		for k, name, shortcut in items:
			if k is None:
				menu.AddItem(BSeparatorItem())
			else:
				msg = BMessage(k)
				item = BMenuItem(name, msg)
				
				if shortcut is None:
					pass
				else:
					keyS, comm = shortcut
					item.SetShortcut(keyS, comm)
				menu.AddItem(item)
				self.mkey[k] = name
		
		self.bar.AddItem(menu)
		self.AddChild(self.bar)
		
		x, barheight = self.bar.GetPreferredSize()
		nbounds = (l, t + barheight, r, b)
		l,t,r,b = nbounds
		
		# Build Splitters
		# 0 = vertical split | 1 = Horizontal split		
		split1 = Splitter(nbounds, 'split',0)
		split2 = Splitter(nbounds, 'split2', 1)
		
		# Build Views
		self.mainList = listy((l,t,r,b), 'tl')
		self.mainList.SetSelectionMessage(BMessage(1009))
		self.confBox = mybox((l+40, t + 80, r - 170, b - 55), 'conf', 'Settings')
		self.appList = listy((l+20, t + 20, r - 175, b - 55), 't32')
		self.appList.SetInvocationMessage(BMessage(1010))
		scrollview1 = scrolly('Fancy Stuff', self.appList, 0,1)
	
		scrollview3 = scrolly('v3', self.mainList, 0, 1)
		self.AddChild(split1)
		
		# 1 = view1 is anchored | 2 = view2 is anchored
		# 25 = view1 takes 25% of thew window width or height
		split1.AddViews(scrollview3, split2, 1, 21)
		split1.SetConstraints(0, 100)
		split2.AddViews(self.confBox, scrollview1, 1, 58, [self.confBox, self.appList])
		split2.SetConstraints(50,50)
		
		
		# setup config box
		l, t, r, b = self.confBox.Bounds()
		self.sysName = texty((l + 5, t + 20, l + 300, t + 35), 'sysname', 'System Name', '', 90.0, 0)
		self.confBox.AddChild(self.sysName)
		
		self.comName = texty((l + 5, t + 50, l + 300, t + 65), 'comname', 'Command', '', 90.0, 0)
		self.confBox.AddChild(self.comName)
		
		self.comParam = texty((l + 5, t + 80, l + 300, t + 95), 'comparam', 'CLI Options', '', 90.0, 0)
		self.confBox.AddChild(self.comParam)
		
		self.parChk = BCheckBox((l + 97, t + 110, l +250, t + 125), 'chk', 'Options after file', BMessage(234))
		self.confBox.AddChild(self.parChk)
		
		self.folderName = texty((l + 5, t + 140, l + 300, t + 155), 'comname', 'Folder', '', 90.0, 0)
		self.confBox.AddChild(self.folderName)
		
		self.folderExt = texty((l + 5, t + 170, l + 300, t + 185), 'comparam', 'File Extensions', '', 90.0, 0)
		self.confBox.AddChild(self.folderExt)
		
		self.saveButton = button((l + 5, t + 200, l + 100, t + 215), 'save', 'Save Settings', 616)
		self.confBox.AddChild(self.saveButton)
		
		for item in self.SYSTEMS.keys():
			
			self.mainList.additem(item)
	
	# The function that runs the command in a new thread
	def comLaunch(self, command, path):
		pathS = os.path.split(path)
		os.chdir(pathS[0])
		print command
		os.system(command)
		
	def update_files(self, item):
		
		self.appList.MakeEmpty()
				
		# get info for that system and populate the configs
		
		#print self.SYSTEMS[item.Text()]
		self.sysName.SetText(self.SYSTEMS[item][0])
		self.comName.SetText(self.SYSTEMS[item][1])
		self.comParam.SetText(self.SYSTEMS[item][2])
		if self.SYSTEMS[item][3] == 1:
			self.parChk.SetValue(1)
			self.optChk = 1
		else:
			self.optChk = 0
		self.folderName.SetText(self.SYSTEMS[item][4])
		self.folderExt.SetText(self.SYSTEMS[item][5])
		self.SELECTED = self.SYSTEMS[item][0]
		
		exts = 0
		# Populate the file list
		# No extension filter
		
		if self.SYSTEMS[item][5] == '':
			
			if self.folderName.Text() == '':
		
				for i in os.listdir('/boot/home/'):
					#if i[-4:] == self.SYSTEMS[item][4]:
					self.appList.additem(i)
			else:
				
				try:
					for i in os.listdir(self.SYSTEMS[item][4]):
					
						self.appList.additem(i)
				except:
					self.about = BAlert('Pickle Problem', 'Invalid Folder Path', 'Ok')
					print self.about.Go()
					return
		else:
			
			exts = self.SYSTEMS[item][5].split(',')
			
			
	
		# just one extension to filter
		if exts == 0:
			if self.folderName.Text() == '':
		
				for i in os.listdir('/boot/home/'):
					if i[-4:] == self.SYSTEMS[item][4]:
						self.appList.additem(i)
			else:
				try:
					for i in os.listdir(self.SYSTEMS[item][3]):
						if i[-4:] == self.SYSTEMS[item][4]:
							self.appList.additem(i)
				except:
					self.about = BAlert('Pickle Problem', 'Invalid Folder Path', 'Ok')
					print self.about.Go()
					return	
		
		# several extensions to filter
		else:
			
			if self.folderName.Text() == '':
				for i in os.listdir('/boot/home/'):
					if i[-4:] in exts:
					
						self.appList.additem(i)
			else:
				try:
				
					for i in os.listdir(self.SYSTEMS[item][4]):
						if i[-4:] in exts:
						
							self.appList.additem(i)
				except:
					self.about = BAlert('Pickle Problem', 'Invalid Folder Path', 'Ok')
					print self.about.Go()
					return
	
	# The switchboard
	def MessageReceived(self, msg):
	
		# Save settings button pressed
		if msg.what == 616:
			sysname = self.sysName.Text()
			if sysname not in self.SYSTEMS.keys():
				self.mainList.additem(sysname)
			
			# get the values and plug it into the SYSTEMS dict			
			confs = [self.sysName.Text(), self.comName.Text(), self.comParam.Text(), self.parChk.Value(), self.folderName.Text(), self.folderExt.Text()]
			self.SYSTEMS[sysname] = confs
			
			# Save settings to file
			try:
				os.remove('settings/systems.cfg')
			except:
				pass
			confp = pickle.dumps(self.SYSTEMS)
			file = open('settings/systems.cfg', 'w')
			file.write(confp)
			file.close()
			
			self.update_files(sysname)
		
		# System selected from Main list
		elif msg.what == 1009:
		
			# Clicked in white space
			if msg.FindInt32('index', 0) == -1:
				self.clear()
			
			else:
				# clear file list
				self.clear()
				self.appList.MakeEmpty()
				
				# get info for that system and populate the configs
				MSGd = msg.FindInt32('index', 0)
				item = self.mainList.ItemAt(MSGd).Text()
								
				self.update_files(item)
		
		# File selected from file list									
		elif msg.what == 1010:
			if msg.FindInt32('index', 0) == -1:
				pass
			else:
				MSGd = msg.FindInt32('index', 0)
				item = self.appList.ItemAt(MSGd)
				itemM = item.Text()
				path = self.SYSTEMS[self.SELECTED][1]
				# build the command from settings and exec in a thread
				if self.optChk:
					command = """%s %s%s %s""" % (self.SYSTEMS[self.SELECTED][1], self.SYSTEMS[self.SELECTED][4], itemM,  self.SYSTEMS[self.SELECTED][2]) 
				
				else:
					command = """%s %s %s%s""" % (self.SYSTEMS[self.SELECTED][1], self.SYSTEMS[self.SELECTED][2], self.SYSTEMS[self.SELECTED][4], itemM) 
				thread.start_new_thread(self.comLaunch, (command, path))
		
		# remove selected system	
		elif msg.what == 1:
			
			count = 0
			
			# find selected system
			while 1:
				# check to see which connection is currently selected
				test = self.mainList.IsItemSelected(count)
				
				# found
				if test:
					# Delete it
					item = self.mainList.ItemAt(count)
					itemM = item.Text()
					del self.SYSTEMS[itemM]
					self.mainList.RemoveItem(count)
					break
				else:
					if count > len(self.SYSTEMS.keys()):
						break
					else:
						count = count+1
			
			# save settings
			try:
				os.remove('settings/systems.cfg')
			except:
				pass
			confp = pickle.dumps(self.SYSTEMS)
			file = open('settings/systems.cfg', 'w')
			file.write(confp)
			file.close()
		
		elif msg.what == 234:
			
			if self.optChk == 0:
				self.optChk = 1
				
			else:	
				self.optChk = 0
				
		# about	
		elif msg.what == 9:
			about_pre = """%s \nby James (Cube-ness) Mayfield, 2003\n\n""" % self.title
			about_post = """Special thanks to edge on beshare for turning me on to this project, as well as helping with the bug hunt.\nMany thanks to all others that helped as well.\n"""
			self.about = BAlert('About pickleLauncher', about_pre + about_post, 'Ok')
			print self.about.Go()
			return
	
	# clear all values/lists
	def clear(self):
		self.appList.MakeEmpty()
		self.sysName.SetText('')
		self.comName.SetText('')
		self.comParam.SetText('')
		self.parChk.SetValue(0)
		self.folderName.SetText('')
		self.folderExt.SetText('')
			
	def QuitRequested(self):
		BApplication.be_app.PostMessage(B_QUIT_REQUESTED)
		return 1

class button(BButton):
	
	def __init__(self, rect, name, label, msg):
		from BFont import be_bold_font, be_plain_font
		be_plain_font.SetSize(10.0)
		BButton.__init__(self, rect, name, label, BMessage(msg), B_FOLLOW_TOP, B_WILL_DRAW)
		

class mybox(BBox):
	def __init__(self, frame, name, label):
		BBox.__init__(self, frame, name, B_FOLLOW_ALL_SIDES, B_FULL_UPDATE_ON_RESIZE|B_FRAME_EVENTS|B_WILL_DRAW|B_FANCY_BORDER)
	
		self.SetViewColor((216,216,216,0))
class scrolly(BScrollView):
	def __init__(self, name, view, hor=1, ver=1):
		BScrollView.__init__(self, name, view, B_FOLLOW_LEFT_RIGHT, B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW|B_FRAME_EVENTS, hor, ver, B_FANCY_BORDER)
		
class listy(BListView):
	def __init__(self, rect, name):
		BListView.__init__(self, rect, name ,B_SINGLE_SELECTION_LIST, B_FOLLOW_TOP_BOTTOM|B_FOLLOW_LEFT_RIGHT,B_WILL_DRAW|B_NAVIGABLE)

	def additem(self, item):
		item_add = BStringItem(item, 0)
		self.AddItem(item_add)

class texty(BTextControl):
	def __init__(self, rect, name, label, info, divider, MSG):
		
		BTextControl.__init__(self, rect, name, label, info, BMessage(MSG), B_FOLLOW_TOP|B_NAVIGABLE)
		self.SetDivider(divider)
	
class MainApp(BApplication.BApplication):
	def __init__(self, sig, rect, title):
		BApplication.BApplication.__init__(self, sig)
		self.rect = rect
		self.title = title
		
	def ReadyToRun(self):
		window = MainWindow(self.rect, self.title)
		window.Show()

AppName = 'pickleLauncher'
Version = '0.4.1'
Creator = 'Cube-ness'
WindowSize = (50, 80, 450, 500)

# Start the App	
ThisApp = MainApp("application/x-vnd." + Creator + '-' + AppName, WindowSize, AppName + ' ' + Version)
ThisApp.Run()